<?php
namespace Teknisa\PED\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class ped00100PedidosFactory {

    protected $connection;

    public function __construct( $entityManager) {
        $this->entityManager       = $entityManager;
    }

    public function getComissaoVend($pedido, $item, $request, $response) {
        try {
            $this->entityManager->getConnection()->beginTransaction();
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'GERA_COMISSAOVEND'); //('GERA_COMISSAOVEND')
            $procedure->addParam(new Param('P_CDFILIAL'    , Param::PARAM_INPUT, $pedido['CDFILIAL']                        , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRPEDIDO'    , Param::PARAM_INPUT, $pedido['NRPEDIDO']                        , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDVENDEDOR'  , Param::PARAM_INPUT, $pedido['CDVENDEDOR']                      , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERADOR'  , Param::PARAM_INPUT, $pedido['CDOPELANPED']                     , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRITEMPED'   , Param::PARAM_INPUT, $item['NRITEMPED']                         , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDSITITEMPED', Param::PARAM_INPUT, $item['IDSITITEMPED']                      , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODUTO'   , Param::PARAM_INPUT, $item['CDPRODUTO']                         , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRPREUNITPED', Param::PARAM_INPUT, $item['VRPREUNITPED']                      , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_OLDCDVENDE'  , Param::PARAM_INPUT, $pedido['CDVENDEDOR']                      , Param::PARAM_TYPE_STR));
            $procedure->execute(); //Invoca a procedure para gerar comissão
            $this->entityManager->getConnection()->commit();
        } catch (\Exception $e) {
            $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
            $this->entityManager->rollback();
            throw $e;

        }


    }

    public function getImpostoItem($pedido, $item, $filial, $request, $response) {
        try {
            $this->entityManager->getConnection()->beginTransaction();
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'IMPOSTO_ITEMPEDIDO');
            $procedure->addParam(new Param('P_EMPRESA'   , Param::PARAM_INPUT, $filial['CDEMPRESA']                         , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_INSCESTA'  , Param::PARAM_INPUT, $filial['CDINSCESTA']                        , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CLIENTE'   , Param::PARAM_INPUT, $pedido['CDCLIENTE']                         , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_FILIDEST'  , Param::PARAM_INPUT, $pedido['CDFILIAL']                        , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_FILIAL'    , Param::PARAM_INPUT, $item['CDFILIAL']                            , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRPEDIDO'  , Param::PARAM_INPUT, $item['NRPEDIDO']                            , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRITEMPED' , Param::PARAM_INPUT, $item['NRITEMPED']                           , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_INIVALID'  , Param::PARAM_INPUT, $item['DTENTITPED']                          , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDTIPOOPER', Param::PARAM_INPUT, $pedido['CDTIPOOPER']                        , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDTIPOOPER', Param::PARAM_INPUT, $pedido['IDENTRSAIDOP']                        , Param::PARAM_TYPE_STR));
            $procedure->execute(); //Invoca a procedure para gerar imposto para os itens
            $this->entityManager->getConnection()->commit();
        } catch (\Exception $e) {
            $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
            $this->entityManager->rollback();
            throw $e;
        }
    }

}