<?php
namespace Teknisa\PED\Modules\Pedidos\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Doctrine\ORM\Query\ResultSetMapping;

class ped00100PedidosFactory {

    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection       = $entityManager->getConnection();
        $this->entityManager    = $entityManager;
    }
    
    
    public function insertVendedorPedAux($params) {
        try {

            $sql = "INSERT INTO VENDPEDAUX (CDFILIAL, NRPEDIDO, CDVENDEDOR) VALUES (:P_CDFILIAL, :P_NRPEDIDO, :P_CDVENDEDOR)";
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            
        }   catch (\Exception $e) {
                
                throw $e;
                
        }
        
    }
    
    public function deleteVendedorPedAux($params) {
        
        try {

            $sql = "DELETE FROM VENDPEDAUX WHERE CDFILIAL = :P_CDFILIAL AND NRPEDIDO = :P_NRPEDIDO AND CDVENDEDOR = :P_CDVENDEDOR";
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            
        }   catch (\Exception $e) {
                
                throw $e;
                
        }
        
    }
    
    public function deleteAllVendPedAux($params) {

        try {
            $this->connection->beginTransaction();
            $sql = "DELETE FROM VENDPEDAUX WHERE CDFILIAL = :P_CDFILIAL AND NRPEDIDO = :P_NRPEDIDO";
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $this->connection->commit();
            return ['done'];
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function selectVendPedAux($params) {
            
        try {
            $sql = "SELECT CDVENDEDOR FROM VENDPEDAUX WHERE CDFILIAL = :P_CDFILIAL AND NRPEDIDO = :P_NRPEDIDO";
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            
            $result = $std->fetchAll();
            return $result;
        }   catch (\Exception $e) {
            throw $e;
        }
    }

    public function getComissaoVend($pedido, $item, $request, $response) {

       try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'GERA_COMISSAOVEND');           //('GERA_COMISSAOVEND')
            $procedure->addParam(new Param('P_CDFILIAL'    , Param::PARAM_INPUT, $pedido['CDFILIAL']               , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRPEDIDO'    , Param::PARAM_INPUT, $pedido['NRPEDIDO']               , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDVENDEDOR'  , Param::PARAM_INPUT, $pedido['CDVENDEDOR']             , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERADOR'  , Param::PARAM_INPUT, $pedido['CDOPELANPED']            , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRITEMPED'   , Param::PARAM_INPUT, $item['NRITEMPED']                , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDSITITEMPED', Param::PARAM_INPUT, $item['IDSITITEMPED']             , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODUTO'   , Param::PARAM_INPUT, $item['CDPRODUTO']                , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRPREUNITPED', Param::PARAM_INPUT, $item['VRPREUNITPED']             , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_OLDCDVENDE'  , Param::PARAM_INPUT, null                              , Param::PARAM_TYPE_STR));
            
            $procedure->execute(); //Invoca a procedure para gerar comissão
        } catch (\Exception $e) {
            $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
            throw $e;
        }
    }

    public function getImpostoItem($pedido, $item, $request, $response, $idmobile) {
        try {
            // $this->entityManager->getConnection()->beginTransaction();
            
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'IMPOSTO_ITEMPEDIDO');
            $procedure->addParam(new Param('P_EMPRESA'   , Param::PARAM_INPUT,  $pedido['CDEMPRESA']                                                                                , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_INSCESTA'  , Param::PARAM_INPUT,  $pedido['CDINSCESTA']                                                                               , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CLIENTE'   , Param::PARAM_INPUT,  $pedido['CDCLIENTE']                                                                                , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_FILIDEST'  , Param::PARAM_INPUT,  $pedido['CDFILIALPED'] = isset($pedido['CDFILIALPED']) ? $pedido['CDFILIALPED'] : ''                , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_FILIAL'    , Param::PARAM_INPUT,  $item['CDFILIAL']                                                                                   , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRPEDIDO'  , Param::PARAM_INPUT,  $item['NRPEDIDO']                                                                                   , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRITEMPED' , Param::PARAM_INPUT,  $item['NRITEMPED']                                                                                  , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_INIVALID'  , Param::PARAM_INPUT,  $item['DTENTITPED']                                                                                 , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDTIPOOPER', Param::PARAM_INPUT,  $pedido['CDTIPOOPER']                                                                               , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDTIPOOPER', Param::PARAM_INPUT,  $pedido['IDORIGDESTOP']                                                                             , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDMOBILE'  , Param::PARAM_INPUT,  $idmobile                                                                                           , Param::PARAM_TYPE_STR));
            $procedure->execute(); //Invoca a procedure para gerar imposto para os itens
            // $this->entityManager->getConnection()->commit();
            
        } catch (\Exception $e) {
            // $this->entityManager->rollback();
            throw $e;
        }
    }

    public function getTotalImposto($filial, $nrpedido, $request, $response) {

        try {

            $this->entityManager->getConnection()->beginTransaction();
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'IMPOSTO_PEDIDO');
            $procedure->addParam(new Param('P_FILIAL'   , Param::PARAM_INPUT,  $filial     , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRPEDIDO' , Param::PARAM_INPUT,  $nrpedido   , Param::PARAM_TYPE_STR));
            $procedure->execute(); //Invoca a procedure para gerar imposto para os itens
            
            $this->entityManager->getConnection()->commit();
        } catch (\Exception $e) {
            $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function geraReserva($item, $response) {
        
        try {
            $this->entityManager->getConnection()->beginTransaction();
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'INSERE_RESERPRODPED');
            
            $procedure->addParam(new Param('P_CDFILIAL' , Param::PARAM_INPUT, $item['CDFILIAL']     , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRPEDIDO' , Param::PARAM_INPUT, $item['NRPEDIDO']     , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRITEMPED', Param::PARAM_INPUT, $item['NRITEMPED']    , Param::PARAM_TYPE_STR));
            // $procedure->addParam(new Param('P_TODOS'    , Param::PARAM_INPUT, $nrpedido          , Param::PARAM_TYPE_STR));
            
            $procedure->execute(); 
            $this->entityManager->getConnection()->commit();

            
        } catch (\Exception $e) {
            $response->setError(new Error("Error. ". $e->getMessage(), 500));
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function returnImposto($CDFILIAL, $NRPEDIDO, $NRORG) {
        try {

            $sql = "SELECT VRIMPOSTOITPED FROM IMPITEMPEDIDO
                    WHERE CDFILIAL = :CDFILIAL
                    AND NRPEDIDO = :NRPEDIDO
                    AND NRORG = :NRORG";

            $params = array('CDFILIAL' => $CDFILIAL,
                            'NRPEDIDO' => $NRPEDIDO,
                            'NRORG' => $NRORG);
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch (\Exception $e) {
            throw $e;
        }
    }

        const GET_VALORES_PEDIDO  ="SELECT 
                                        I.NRORG,
                                        I.CDFILIAL,
                                        I.NRPEDIDO,
                                        (I.VRITEM + NVL(E.VRENCARGO,0) + NVL(IM.VRIMPOSTO,0)) VRTOTALPED,
                                        I.VRITEM AS VRTOTPEDLIQ,
                                        ((I.VRITEM + NVL(E.VRENCARGO,0) + NVL(IM.VRIMPOSTO,0)) - I.VRITEM) VALORIMPOSTO
                                    FROM PEDIDO P,
                                        (SELECT I.NRORG,
                                            I.CDFILIAL,
                                            I.NRPEDIDO,
                                            SUM((I.QTITEMPED * I.VRPREUNITPED) - (I.VRDESITPED)) VRITEM
                                        FROM ITEMPEDIDO I
                                        GROUP BY I.NRORG,
                                            I.NRPEDIDO,
                                            I.CDFILIAL
                                        ) I,
                                        (SELECT E.NRORG,
                                            E.CDFILIAL,
                                            E.NRPEDIDO,
                                            SUM(
                                                CASE EN.IDENCARGO
                                                WHEN 'A'
                                                    THEN TO_CHAR(NVL(E.VRENCARGOPED,0))
                                                WHEN 'D'
                                                    THEN TO_CHAR(NVL(E.VRENCARGOPED,0) * -1)
                                                ELSE '0'
                                                END
                                            ) VRENCARGO
                                        FROM ENCARGOPED E,
                                            ENCARGO EN
                                        WHERE E.CDENCARGO = EN.CDENCARGO
                                            AND E.NRORG       = EN.NRORG
                                        GROUP BY E.NRORG,
                                            E.CDFILIAL,
                                            E.NRPEDIDO
                                        ) E,
                                        (SELECT IP.NRORG,
                                            IP.CDFILIAL,
                                            IP.NRPEDIDO,
                                            SUM(
                                                CASE IM.IDTRATIMPO
                                                WHEN 'A'
                                                    THEN TO_CHAR(NVL(IP.VRIMPOSTOITPED,0))
                                                WHEN 'D'
                                                    THEN TO_CHAR(NVL(IP.VRIMPOSTOITPED,0) * -1)
                                                    ELSE '0'
                                                    END
                                            ) VRIMPOSTO
                                        FROM IMPOSTO IM,
                                            IMPITEMPEDIDO IP
                                        WHERE IP.CDIMPOSTO = IM.CDIMPOSTO
                                            AND IP.NRORG       = IM.NRORG
                                        GROUP BY IP.NRORG,
                                            IP.CDFILIAL,
                                            IP.NRPEDIDO
                                        ) IM
                                    WHERE P.CDFILIAL        = I.CDFILIAL (+)
                                        AND P.NRPEDIDO              = I.NRPEDIDO (+)
                                        AND P.NRORG                 = I.NRORG (+)
                                        AND P.CDFILIAL              = E.CDFILIAL (+)
                                        AND P.NRPEDIDO              = E.NRPEDIDO (+)
                                        AND P.NRORG                 = E.NRORG (+)
                                        AND P.CDFILIAL              = IM.CDFILIAL (+)
                                        AND P.NRPEDIDO              = IM.NRPEDIDO (+)
                                        AND P.NRORG                 = IM.NRORG (+)
                                        AND P.CDFILIAL              = :P_CDFILIAL
                                        AND P.NRPEDIDO              = :P_NRPEDIDO";

    public function getValoresPedido($cdfilial, $nrpedido){
        try {
            $params = array(
                'P_CDFILIAL' => $cdfilial,
                'P_NRPEDIDO' => $nrpedido
            );
            return $this->entityManager->getConnection()->fetchAll(self::GET_VALORES_PEDIDO, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

        const GET_MAX_NRDADOADICPED = "SELECT NVL(MAX(NRDADOADICPED), '001') AS MAX
                                    FROM DADOSADICPED
                                    WHERE CDFILIAL = :P_CDFILIAL
                                        AND NRPEDIDO = :P_NRPEDIDO";

    public function getMaxNrdadoadicped($cdfilial, $nrpedido){
        try {
            $params = array(
                'P_CDFILIAL' => $cdfilial,
                'P_NRPEDIDO' => $nrpedido
            );
            return $this->entityManager->getConnection()->fetchAll(self::GET_MAX_NRDADOADICPED, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
        const INSERT_HISITEMPED = "INSERT INTO HISITEMPED
                                (IDSITANTHIST, IDSITATUHIST, DTHRHIST, CDFILIAL, NRPEDIDO, NRITEMPED, NRORG)
                              VALUES (:P_IDSITANTHIST,
                                    :P_IDSITATUHIST,
                                    :P_DTHRHIST,
                                    :P_CDFILIAL,
                                    :P_NRPEDIDO,
                                    :P_NRITEMPED,
                                    :P_NRORG)";
    
    public function insertHist($historico) {
        try {
            $params = array(
                'P_IDSITANTHIST'=>  $historico['IDSITANTHIST'],
                'P_IDSITATUHIST'=>  $historico['IDSITATUHIST'],
                'P_DTHRHIST'=> $historico['DTHRHIST'],
                'P_CDFILIAL'=> $historico['CDFILIAL'],
                'P_NRPEDIDO'=> $historico['NRPEDIDO'],
                'P_NRITEMPED'=> $historico['NRITEMPED'],
                'P_NRORG'=> $historico['NRORG']
            );
            return $this->entityManager->getConnection()->executeQuery(self::INSERT_HISITEMPED, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
            const INSERT_ITEMPEDIDOTRAB = "INSERT INTO ITEMPEDIDOTRAB
                                (CDFILIAL,NRPEDIDO, NRITEMPED, DTENTITPED, CDPRODUTO, QTITEMPED, VRPREUNITPED, VRDESITPED)
                              VALUES (:P_CDFILIAL,
                                      :P_NRPEDIDO,
                                      :P_NRITEMPED,
                                      :P_DTENTITPED,
                                      :P_CDPRODUTO,
                                      :P_QTITEMPED,
                                      :P_VRPREUNITPED,
                                      :P_VRDESITPED)";
    
    public function insereItemtrab($item, $response) {
        try {
            $params = array(
                'P_CDFILIAL' => $item['CDFILIAL'],
                'P_NRPEDIDO' => $item['NRPEDIDO'],
                'P_NRITEMPED' => $item['NRITEMPED'],
                'P_DTENTITPED' => $item['DTENTITPED'],
                'P_CDPRODUTO' => $item['CDPRODUTO'],
                'P_QTITEMPED' => $item['QTITEMPED'],
                'P_VRPREUNITPED' => $item['VRPREUNITPED'],
                'P_VRDESITPED' => $item['VRDESITPED'],
                'P_NRORG'=> $item['NRORG']
            );
            
            return $this->entityManager->getConnection()->executeQuery(self::INSERT_ITEMPEDIDOTRAB, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    
    public function consultaPendencia($params) {
        
        $CONSULTA_PENDENCIA = "SELECT
                                   NF.CDCLIENTE,
                                   NF.CDFILIAL,
                                   NF.NRLANCTONF,
                                   NF.NRNOTAFISC,
                                   CO.DTATUAVENREC,
                                   CO.VRATUAREC
                                FROM
                                    CONTARECEBER   CO,
                                    NOTAFISCAL     NF,
                                    BOLETRECEBER   BO,
                                    CLIENTE        C,
                                    PARAMPEDIDO    P
                                WHERE
                                    ( NF.CDCLIENTE = :P_CDCLIENTE )
                                    AND ( CO.CDFILIAL = NF.CDFILIAL )
                                    AND ( CO.NRLANCTONF = NF.NRLANCTONF )
                                    AND ( CO.NRLANBOLREC = BO.NRLANBOLREC )
                                    AND ( CO.CDFILIAL = BO.CDFILIAL )
                                    AND ( P.CDFILIAL = :P_CDFILIAL )
                                    AND ( C.CDCLIENTE = :P_CDCLIENTE )
                                    AND ( BO.DTVENBOLREC < ( SYSDATE - NVL(C.NRDIAPFINAPR, P.NRDIAPENDFIN) ) )
                                    AND ( CO.IDSTATREC IN (
                                        'A',
                                        'D'
                                    ) )
                                    AND ( NF.IDSTATUS IN (
                                        'A',
                                        'E'
                                    ) )";
        $stmt = $this->connection->prepare($CONSULTA_PENDENCIA);
        $stmt->execute($params);
        $result = $stmt->fetchAll();
        return $result;
    }
    
    public function consultaIdPendFin($params) {
        
        $CONSULTA_IDPEND_PARAMPEDIDO = "SELECT
                                        NVL(C.IDPFINAPRPED, P.IDPENDFINAN) AS IDPENDFINAN
                                    FROM
                                        PARAMPEDIDO   P,
                                        CLIENTE       C
                                    WHERE
                                        ( P.CDFILIAL = :P_CDFILIAL )
                                        AND ( C.CDCLIENTE = :P_CDCLIENTE )";
                                
        $stmt = $this->connection->prepare($CONSULTA_IDPEND_PARAMPEDIDO);
        $stmt->execute($params);
        $result = $stmt->fetch();
        return $result;
    }
    
    public function consultaIdPendFinGroup($params) {
        
        $CONSULTA_PARAMPEDIDO = "SELECT
                                        NVL(C.IDPFINAPRPED, P.IDPENDFINAN) AS IDPENDFINAN
                                    FROM
                                        PARAMPEDIDO   P,
                                        CLIENTE       C
                                    WHERE
                                        ( P.CDFILIAL IN (:P_CDFILIAL) )
                                        AND ( C.CDCLIENTE IN (:P_CDCLIENTE) )";
        $rsm = new ResultSetMapping;        
        $query = $this->entityManager->createNativeQuery($CONSULTA_PARAMPEDIDO,$rsm);
        $query->setParameter('P_CDFILIAL',$params['P_CDFILIAL']);
        $query->setParameter('P_CDCLIENTE',$params['P_CDCLIENTE']);
        $result = $query->getResult();

        return $result;
    }
    const DELETE_ITEMPEDIDOTRAB = "DELETE FROM ITEMPEDIDOTRAB
                                        WHERE 
                                            CDFILIAL = :P_CDFILIAL AND
                                            NRPEDIDO = :P_NRPEDIDO";
    
    public function deleteItemtrab($item, $response) {
        try {
            $params = array(
                'P_CDFILIAL' => $item['CDFILIAL'],
                'P_NRPEDIDO' => $item['NRPEDIDO']
            );
            return $this->entityManager->getConnection()->executeQuery(self::DELETE_ITEMPEDIDOTRAB, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    const DELETE_IMPITEMPEDIDOTRAB = "DELETE FROM IMPITEMPEDIDOTRAB
                                        WHERE 
                                            CDFILIAL = :P_CDFILIAL AND
                                            NRPEDIDO = :P_NRPEDIDO";

    public function deleteImpitempedidotrab($pedido, $response) {
        try {
            $params = array(
                'P_CDFILIAL' => $pedido['CDFILIAL'],
                'P_NRPEDIDO' => $pedido['NRPEDIDO']
            );
            return $this->entityManager->getConnection()->executeQuery(self::DELETE_IMPITEMPEDIDOTRAB, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getImpitempedidotrab($pedido) {
        try {
            $sql = "SELECT IP.CDFILIAL,
                        IP.NRPEDIDO,
                        IP.NRITEMPED,
                        SUM(
                        CASE IM.IDTRATIMPO
                          WHEN 'A'
                          THEN TO_CHAR(NVL(IP.VRIMPOSTOITPED,0))
                          WHEN 'D'
                          THEN TO_CHAR(NVL(IP.VRIMPOSTOITPED,0) * -1)
                          ELSE '0'
                        END ) VRIMPOSTOITPED
                      FROM IMPOSTO IM,
                        IMPITEMPEDIDOTRAB IP
                      WHERE IP.CDIMPOSTO = IM.CDIMPOSTO
                      AND IP.CDFILIAL = :P_CDFILIAL
                      AND IP.NRPEDIDO = :P_NRPEDIDO 
                      GROUP BY IP.CDFILIAL,
                        IP.NRPEDIDO,
                        IP.NRITEMPED
                    ";
            $params = array('P_CDFILIAL' => $pedido['CDFILIAL'],
                            'P_NRPEDIDO' => $pedido['NRPEDIDO']);
                
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    const INSERT_ENCARGOTRAB = "INSERT INTO ENCARGOPEDTRAB
            (CDFILIAL, NRPEDIDO, CDENCARGO, VRENCARGOPED)
              VALUES (:P_CDFILIAL,
                  :P_NRPEDIDO,
                  :P_CDENCARGO,
                  :P_VRENCARGOPED)";
    public function saveEncargotrab($encargo) {
        try {
            $params = array(
                'P_CDFILIAL' => $encargo['CDFILIAL'],
                'P_NRPEDIDO' => $encargo['NRPEDIDO'],
                'P_CDENCARGO' => $encargo['CDENCARGO'],
                'P_VRENCARGOPED' => $encargo['VRENCARGOPED']
            );
            return $this->entityManager->getConnection()->executeQuery(self::INSERT_ENCARGOTRAB, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
        const DELETE_ENCARGOTRAB = "DELETE FROM ENCARGOPEDTRAB
                                        WHERE 
                                            CDFILIAL = :P_CDFILIAL AND
                                            NRPEDIDO = :P_NRPEDIDO";

    public function deleteEncargotrab($pedido) {
        try {
            $params = array(
                'P_CDFILIAL' => $pedido['CDFILIAL'],
                'P_NRPEDIDO' => $pedido['NRPEDIDO']
            );
            return $this->entityManager->getConnection()->executeQuery(self::DELETE_ENCARGOTRAB, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getImpostosped($pedido) {

        $sql = "SELECT CDFILIAL,
                       NRPEDIDO,
                       CDIMPOSTO,
                       SUM(VRIMPOSTOITPED) AS VRIMPOSTOTAL,
                       SUM(VRBASECALCITPED) AS VRBASETOTAL
                FROM IMPITEMPEDIDO
                WHERE CDFILIAL   = :P_CDFILIAL AND
                       NRPEDIDO  = :P_NRPEDIDO
                GROUP BY CDFILIAL, NRPEDIDO, CDIMPOSTO
        ";
        $params = array('P_CDFILIAL' => $pedido['CDFILIAL'],
                        'P_NRPEDIDO' => $pedido['NRPEDIDO']);
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function insereImpostototped($pedido) {
        $params = array('P_CDFILIAL' => $pedido['CDFILIAL'],
                        'P_NRPEDIDO' => $pedido['NRPEDIDO']);
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
        const INSERT_IMPOSNFTOTPED = "INSERT INTO IMPOSNFTOTPED
                        (CDFILIAL, NRPEDIDO, CDIMPOSTO,
                         VRTOTIMPPED, VRTOTBCALCPED)
                    VALUES
                        (:CDFILIAL, :NRPEDIDO, :CDIMPOSTO,
                         :VRTOTIMPPED, :VRTOTBCALCPED)
                    ";
    public function saveImpostototped($imp) {
        try {
            $params = array(
                'CDFILIAL' => $imp['CDFILIAL'],
                'NRPEDIDO' => $imp['NRPEDIDO'],
                'CDIMPOSTO' => $imp['CDIMPOSTO'],
                'VRTOTIMPPED' => $imp['VRIMPOSTOTAL'],
                'VRTOTBCALCPED' => $imp['VRBASETOTAL']
            );
            return $this->entityManager->getConnection()->executeQuery(self::INSERT_IMPOSNFTOTPED, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getModpgtoPed($params) {

        try {
            $sql = "SELECT CDMODPGTO FROM PEDIDO WHERE CDFILIAL = :P_CDFILIAL AND NRPEDIDO = :P_NRPEDIDO";
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    public function getRotaEnt($idorig, $filialdest, $cdcliente) {
        try {
            $sql = "SELECT CL.CDROTATRANS, RO.NMROTATRANS, CL.IDROTAPRINCLI AS IDROTAPRINC
                    FROM CLIENROTA CL, ROTATRANS RO
                    WHERE CL.CDROTATRANS  = RO.CDROTATRANS
                        AND CL.CDCLIENTE    = :P_CDCLIENTE
                        AND :P_IDORIGDESTOP = 'C'
                    UNION
                    SELECT FL.CDROTATRANS, RO.NMROTATRANS, FL.IDROTAPRINCFILI  AS IDROTAPRINC
                    FROM FILIALROTA FL, ROTATRANS RO
                    WHERE FL.CDROTATRANS  = RO.CDROTATRANS
                        AND FL.CDFILIROTA   = :P_CDFILIROTA 
                        AND :P_IDORIGDESTOP = 'L'";
            
            $params = array(
                'P_IDORIGDESTOP'   => $idorig,
                'P_CDFILIROTA'     => $filialdest,
                'P_CDCLIENTE'      => $cdcliente
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getDataEntrega($filial, $dtbase) {
        try {
            $sql = "SELECT DATAUTIL(:P_CDFILIAL, :P_DTBASE, NRDIASENTPED) AS DTENTREGA FROM PARAMPEDIDO WHERE CDFILIAL = :P_CDFILIAL";
            
            $params = array(
                'P_CDFILIAL' => $filial,
                'P_DTBASE' => $dtbase
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function tabelaItemPedido($pedido) {
        try {
            $sql = "SELECT IP.QTITEMPED, IP.VRPREUNITPED, P.NMPRODUTO 
                    FROM ITEMPEDIDO IP, PRODUTO P
                    WHERE IP.CDPRODUTO = P.CDPRODUTO
                        AND CDFILIAL = :P_CDFILIAL
                        AND NRPEDIDO = :P_NRPEDIDO";
            
            $params = array(
                'P_CDFILIAL' => $pedido['CDFILIAL'],
                'P_NRPEDIDO' => $pedido['NRPEDIDO']
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    public function endEntregaPed($pedido) {
        try {
            $sql = "SELECT * FROM DADOSENDPED 
                            WHERE IDTPENDECLIE IN ('P','T')
                            AND CDCLIENTE = :P_CDCLIENTE
                            AND NRPEDIDO = :P_NRPEDIDO
                            AND CDFILIAL = :P_CDFILIAL";
            
            $params = array(
                'P_NRPEDIDO' => $pedido['NRPEDIDO'],
                'P_CDFILIAL' => $pedido['CDFILIAL'],
                'P_CDCLIENTE' => $pedido['CDCLIENTE'],

            ); 
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    public function getSmtp($pedido){
        try {
            $sql = "SELECT DSLOGINSMTP, CDSENHEMAILF, DSSMTPEMAILF 
                                FROM ENDEFILI E, PEDIDO P 
                                WHERE NRPEDIDO = :P_NRPEDIDO 
                                AND P.CDFILIAL = :P_CDFILIAL
                                AND P.CDFILIAL = E.CDFILIAL";
            
            $params = array(
                'P_NRPEDIDO' => $pedido['NRPEDIDO'],
                'P_CDFILIAL' => $pedido['CDFILIAL']
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function deleteITEMPEDIDO($params) {
        try {
            $sql = "DELETE FROM ITEMPEDIDO 
                    WHERE CDFILIAL = :P_CDFILIAL 
                    AND NRPEDIDO = :P_NRPEDIDO";
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }   catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function deletePEDIDO($params) {
        try {
            $sql = "DELETE FROM PEDIDO 
                    WHERE CDFILIAL = :P_CDFILIAL 
                    AND NRPEDIDO = :P_NRPEDIDO";
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }   catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaProdutoContEst($cdfilial, $cdproduto){
        try {
            $sql = "SELECT
                       P.IDCNTRESTOQ
                    FROM
                       PRODFILI P, PRODUTO M
                    WHERE
                       (P.IDCNTRESTOQ = 'S') AND
                       (P.CDFILIAL  = :P_CDFILIAL) AND
                       (P.CDPRODUTO = M.CDPRODESTO)AND
                       (M.CDPRODUTO = :P_CDPRODUTO)";
            
            $params = array(
                'P_CDFILIAL'  => $cdfilial,
                'P_CDPRODUTO' => $cdproduto
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getQtDispQtPed($cdfilial, $cdproduto){
        try {
            $sql = "SELECT SUM(IT.QTITEMPED*NVL(PR.VRFATOCONV, 1)) AS QTITEMPED,
                       SUM((IT.QTITEMPED - IT.QTEXPITEMPED)*NVL(PR.VRFATOCONV, 1)) AS QTITEMPEDEXP
                  FROM PEDIDO PE, ITEMPEDIDO IT, PRODUTO PR, TIPOOPERAC TP
                 WHERE PE.NRPEDIDO     = IT.NRPEDIDO
                   AND IT.CDPRODUTO    = PR.CDPRODUTO
                   AND IT.CDFILIAL     = PE.CDFILIAL
                   AND PE.IDSITPEDIDO  IN ('1', '2', '6')
                   AND IT.IDSITITEMPED IN ('1','2','3','7')
                   AND PR.CDPRODESTO   = :P_CDPRODUTO
                   AND PE.CDFILIAL     = :P_CDFILIAL
                   AND PE.IDTIPOPEDIDO = '1'
                   AND PE.CDTIPOOPER   = TP.CDTIPOOPER
                   AND PE.IDENTRSAIDOP = TP.IDENTRSAIDOP
                   AND NVL(TP.IDATUESTTPOP, 'S') = 'S'";
            
            $params = array(
                'P_CDPRODUTO' => $cdproduto,
                'P_CDFILIAL'  => $cdfilial
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function qtdeDisponivel($cdfilial, $cdproduto, $idctrldistrib, $idutilalmox){
        try {
            $sql = "SELECT SUM(RP.QTTOTALESTOQ) QTTOTALESTOQ, SUM(RP.QTDISPONIVEL) QTDISPONIVEL,
                           SUM(RP.QTRESPRODPED) QTRESPRODPED
                      FROM LOCALESTOQUE LE, RESERVAPED RP, PRODUTO PR, LOTE LT, PRODFILI PF
                     WHERE ((RP.CDFILIAL    = (:P_CDFILIAL) OR (RP.CDFILIAL = (SELECT CDFILDEPOSITO FROM PARAMPEDIDO WHERE CDFILIAL = :P_CDFILIAL AND IDUTDEPOSITO = 'S'))))
                       AND PR.CDPRODUTO    = :P_CDPRODUTO
                       AND RP.CDPRODUTO    = PR.CDPRODESTO
                       AND RP.CDFILIAL     = LE.CDFILIAL
                       AND RP.CDALMOXARIFE = LE.CDALMOXARIFE
                       AND RP.CDLOCALESTOQ = LE.CDLOCALESTOQ
                       AND RP.CDFILIAL     = PF.CDFILIAL
                       AND RP.CDPRODUTO    = PF.CDPRODUTO
                       AND RP.CDALMOXARIFE = NVL(:P_CDALMOXARIFE, RP.CDALMOXARIFE)
                       AND RP.CDLOCALESTOQ = NVL(:P_CDLOCALESTOQ, RP.CDLOCALESTOQ)
                       AND ((PF.IDUTCTRLLOTE = 'S') AND (TRUNC(LT.DTVALILOTE) >= TRUNC(SYSDATE))
                            OR ((PF.IDUTCTRLLOTE = 'N')))
                       AND RP.NRLOTEESTQ   = LT.NRLOTEESTQ (+)
                       AND RP.NRSUBLOTE    = LT.NRSUBLOTE  (+)
                       AND RP.CDPRODUTO    = LT.CDPRODUTO  (+)
                       AND RP.CDFILIAL     = LT.CDFILIAL   (+)
                       AND ((:P_IDUTILALMOX = 'N')
                            OR ((LE.IDDISPCONPED = 'S') AND
                                (LE.CDLOCALESTOQ = (SELECT NVL(CDLOCALPRCLI, ' ')
                                                      FROM PRODFILI PF
                                                     WHERE (PF.CDFILIAL    = (:P_CDFILIAL) OR (PF.CDFILIAL = (SELECT CDFILDEPOSITO FROM PARAMPEDIDO WHERE CDFILIAL = :P_CDFILIAL AND IDUTDEPOSITO = 'S')))
                                                       AND PF.CDPRODUTO    = :P_CDPRODUTO
                                                       AND PF.CDALMOXPRCLI = LE.CDALMOXARIFE))
                            )
                           )
                     UNION
                    SELECT SUM(RP.QTTOTALESTOQ) QTTOTALESTOQ, SUM(RP.QTDISPONIVEL) QTDISPONIVEL,
                           SUM(RP.QTRESPRODPED) QTRESPRODPED
                      FROM LOCALESTOQUE LE, RESERVAPED RP, PRODUTO PR, LOTE LT, PRODFILI PF
                     WHERE ((RP.CDFILIAL    = (:P_CDFILIAL) OR (RP.CDFILIAL = (SELECT CDFILDEPOSITO FROM PARAMPEDIDO WHERE CDFILIAL = :P_CDFILIAL AND IDUTDEPOSITO = 'S'))))
                       AND PR.CDPRODUTO    = :P_CDPRODUTO
                       AND RP.CDPRODUTO    = PR.CDPRODESTO
                       AND RP.CDFILIAL     = LE.CDFILIAL
                       AND RP.CDALMOXARIFE = LE.CDALMOXARIFE
                       AND RP.CDLOCALESTOQ = LE.CDLOCALESTOQ
                       AND RP.CDFILIAL     = PF.CDFILIAL
                       AND RP.CDPRODUTO    = PF.CDPRODUTO
                       AND :P_IDTERC = 'S'
                       AND RP.CDALMOXARIFE = NVL(:P_CDALMOXARIFE, RP.CDALMOXARIFE)
                       AND RP.CDLOCALESTOQ = NVL(:P_CDLOCALESTOQ, RP.CDLOCALESTOQ)
                       AND ((PF.IDUTCTRLLOTE = 'S') AND (TRUNC(LT.DTVALILOTE) >= TRUNC(SYSDATE))
                            OR ((PF.IDUTCTRLLOTE = 'N') AND (LT.DTVALILOTE IS NULL)))
                       AND RP.NRLOTEESTQ   = LT.NRLOTEESTQ (+)
                       AND RP.NRSUBLOTE    = LT.NRSUBLOTE  (+)
                       AND RP.CDPRODUTO    = LT.CDPRODUTO  (+)
                       AND ((:P_IDUTILALMOX = 'N') OR ((LE.IDDISPCONPED = 'S')))";
            
            $params = array(
                'P_CDPRODUTO' => $cdproduto,
                'P_CDFILIAL'  => $cdfilial,
                'P_IDTERC'  => $idctrldistrib,
                'P_IDUTILALMOX'  => $idutilalmox,
                'P_CDALMOXARIFE'  => '',
                'P_CDLOCALESTOQ'  => ''
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    
    public function replicaPedido($pedido, $prxPedido, $cdoperador) {
        
        try {

            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'REPLICA_PEDIDO');
            $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT,  $pedido['CDFILIAL']       , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRPEDIDONEW',     Param::PARAM_INPUT,  $prxPedido['NOVOCODIGO']  , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDCLIENTE' ,      Param::PARAM_INPUT,  $pedido['CDCLIENTE']      , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILIDEST',      Param::PARAM_INPUT,  $pedido['CDFILIALPED']    , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DATAPEDIDO',      Param::PARAM_INPUT,  $pedido['DTPEDIDO']       , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DATAENTREGA',     Param::PARAM_INPUT,  $pedido['DTENTPED']       , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DATAVALIDADE',    Param::PARAM_INPUT,  $pedido['DTVALIPED']      , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRORDCOMPRA' ,    Param::PARAM_INPUT,  $pedido['NRORDEMCMPPED']  , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILIALPEDORIG', Param::PARAM_INPUT,  $pedido['CDFILIAL']       , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRPEDIDO',        Param::PARAM_INPUT,  $pedido['NRPEDIDO']       , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERADOR',      Param::PARAM_INPUT,  $cdoperador               , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMENSERRO',      Param::PARAM_OUTPUT, ''                        , Param::PARAM_TYPE_STR, 1000));
            return $procedure->execute(); 
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function qtdeDispEstqReserva($reserva){
        try {
            $sql = "SELECT RE.CDPRODUTO,
                          RE.CDALMOXARIFE,
                          RE.CDLOCALESTOQ,
                          RE.NRLOTEESTQ,
                          RE.NRSUBLOTE,
                          RP.QTRESPRODPED,
                          RP.NRPEDIDO,
                          RP.NRITEMPED,
                          RP.NRORDPROD,
                          RP.NRSUBORDPROD,
                          RP.QTRESPRODARR,
                          LT.CDFABRIC,
                          FA.NMFABRIC,
                          RE.CDFILIAL,
                          RP.NRRESPRODPED,
                          RE.NRORG,
                          RE.QTDISPONIVEL,
                          LT.DTENTRLOTE,
                          LT.DTVALILOTE,
                          PR.SGUNIDADE
                        FROM RESERVAPED RE
                        INNER JOIN PRODUTO PR
                        ON RE.CDPRODUTO = PR.CDPRODUTO
                        LEFT JOIN RESERPRODPED RP
                        ON RP.CDALMOXARIFE  = RE.CDALMOXARIFE
                        AND RP.CDLOCALESTOQ = RE.CDLOCALESTOQ
                        AND RP.NRLOTEESTQ   = RE.NRLOTEESTQ
                        AND RP.NRSUBLOTE    = RE.NRSUBLOTE
                        AND RP.CDFILIAL     = RE.CDFILIAL
                        AND RP.CDPRODUTO    = RE.CDPRODUTO
                        AND RP.NRPEDIDO     = :P_NRPEDIDO
                        AND RP.NRITEMPED    = :P_NRITEMPED
                        LEFT JOIN LOTE LT
                        ON RE.CDFILIALDEP = LT.CDFILIAL
                        AND RE.NRLOTEESTQ = LT.NRLOTEESTQ
                        AND RE.NRSUBLOTE  = LT.NRSUBLOTE
                        AND RE.CDPRODUTO  = LT.CDPRODUTO
                        LEFT JOIN FABRICANTE FA
                        ON LT.CDFABRIC       = FA.CDFABRIC
                        WHERE RE.CDFILIAL    = :P_CDFILIAL
                        AND RE.CDPRODUTO     = :P_CDPRODUTO
                        AND (RE.QTDISPONIVEL > 0
                        OR RP.QTRESPRODPED   > 0)
                        AND RE.NRLOTEESTQ    = :P_NRLOTEESTQ";
            
            $params = array(
                'P_CDFILIAL'  => $reserva['CDFILIAL'],
                'P_NRPEDIDO'  => $reserva['NRPEDIDO'],
                'P_NRITEMPED'  => $reserva['NRITEMPED'],
                'P_CDPRODUTO'  => $reserva['CDPRODUTO'],
                'P_NRLOTEESTQ'  => $reserva['NRLOTEESTQ']
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function recalculaVrPed($cdfilial, $nrpedido){
        
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'RECALCULA_PRECOPED');
            $procedure->addParam(new Param('P_CDFILIAL'     , Param::PARAM_INPUT, $cdfilial     , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRPEDINICIAL' , Param::PARAM_INPUT, $nrpedido     , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRPEDFINAL'   , Param::PARAM_INPUT, $nrpedido     , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRECOORIG'  , Param::PARAM_INPUT, ''            , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRECODEST'  , Param::PARAM_INPUT, ''            , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDDTATUAL'    , Param::PARAM_INPUT, 'N'           , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRERRO'       , Param::PARAM_OUTPUT,''            , Param::PARAM_TYPE_STR, 1));
            
          return  $procedure->execute();

        
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    
    public function validaPedidoImportado($row){
         try {
            $sql = "SELECT NREXPEDIDO, NRPEDIDO
                      FROM PEDIDO
                     WHERE NREXPEDIDO = :P_NREXPEDIDO
                       AND IDSITPEDIDO <> '4'";
            
            $params = array(
                'P_NREXPEDIDO' => $row
            ); 
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function validaClientPed($row){
         try {
            $sql = "SELECT CDCLIENTE, 
                        NRINSJURCLIE,
                        CDTIPTRANS,
                        IDPGFRETTRANCLI,
                        NRINSJURTRAN,
                        IDTPIJURTRAN, 
                        NMRAZSOCCLIE,
                        CDEMPRESA,
                        CDINSCESTA
                        FROM CLIENTE
                        WHERE NRINSJURCLIE = NVL((RTRIM(:P_CDCLIENTE,' ')),NRINSJURCLIE)"
            ;
            
            $params = array(
                'P_CDCLIENTE' => $row
            ); 
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaModalidadePagamento($row){
        try {
            $sql = "SELECT CDCLIENTE, CDMODPGTO, CDTIPTRANS, IDPGFRETTRANCLI,
                    NRINSJURTRAN, IDTPIJURTRAN
                      FROM CLIENTE
                     WHERE CDCLIENTE = NVL((RTRIM(:P_CDCLIENTE,' ')),CDCLIENTE)";
            
            $params = array(
                'P_CDCLIENTE' => $row,
            ); 
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function validaModalidadePagamentoPadrao($row){
        try {
            $sql = "SELECT IDPERALTPRECO,   CDMODPGTO, CDVENDEDOR,   IDTABPRECO,
                           IDLANCPED,       IDTESTAEST,  IDOBRINFTRAN, NRDIASENTPED, NRMAXITNFPED,
                           SYSDATE AS DATA, IDUTVALIPED, IDOBRTPOPPED, QTDIAVALIPED,  IDUTDESCMAXI,
                           CDTIPOOPER, IDENTRSAIDOP, IDUTRESELOTE, IDEXIBIMPITPED, CDFILIAL
                      FROM PARAMPEDIDO
                     WHERE CDFILIAL = :P_CDFILIAL"
            ;
            
            $params = array(
                'P_CDFILIAL' => $row,
            ); 
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    
    public function validaModalidadePagamentoCadastrada($row){
        try {
            $sql = "SELECT CDMODPGTO, DSMODPGTO, CDFORMPGTO, IDTPFORPGTOMOD,
                           NRPARCPGTOMOD, NRDIAPGFIXMOD, IDUTDIAPAGMOD, VRPERCERENT
                      FROM MODPGTO
                     WHERE CDMODPGTO =  LTRIM(:P_CDMODPGTO)
                     ORDER BY CDMODPGTO"
            ;
            
            $params = array(
                'P_CDMODPGTO' => $row,
            ); 
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaVendedor($row){
        try {
            $sql = "SELECT CDVENDEDOR
                      FROM VENDEDOR
                     WHERE CDVENDEDOR = NVL(:P_CDVENDEDOR, CDVENDEDOR)"
            ;
            
            $params = array(
                'P_CDVENDEDOR' => $row,
            ); 
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaTabelaPrecoPrincipal($row){
        try {
            $sql = "SELECT CDPRECOBASE
                      FROM PRECOBASECLI
                     WHERE CDCLIENTE = :P_CDCLIENTE 
                       AND IDPRBASEPRIN = :P_IDPRBASEPRIN"
            ;
            
            $params = array(
                'P_CDCLIENTE' => $row,
                'P_IDPRBASEPRIN' => 'S',
            ); 
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }    
    
    public function validaTabelaPrecoSecundaria($row){
        try {
            $sql = "SELECT CDPRECOBASE
                      FROM PRECOBASECLI
                     WHERE CDCLIENTE = :P_CDCLIENTE 
                       AND IDPRBASEPRIN = :P_IDPRBASEPRIN"
            ;
            
            $params = array(
                'P_CDCLIENTE' => $row,
                'P_IDPRBASEPRIN' => 'N',
            ); 
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }    

    public function validaTabelaPrecoVendedor($row){
        try {
            $sql = "SELECT CDPRECOBASE 
                      FROM VENDEDOR
                     WHERE CDVENDEDOR = :P_CDVENDEDOR"
            ;
            
            $params = array(
                'P_CDVENDEDOR' => $row
            ); 
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }    
    
    public function validaTabelaPrecoPreCadastrada($row){
        try {
            $sql = "SELECT CDPRECOBASE 
                      FROM PRECOBASE
                     WHERE CDPRECOBASE = :P_CDPRECOBASE"
            ;
            
            $params = array(
                'P_CDPRECOBASE' => $row
            ); 
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }    
    
     public function validaItem($row){
        try {
            $sql = "SELECT
                       CDPRODUTO, CDNVPRODUTO, CDARVPROD,SGUNIDADE
                    FROM
                       PRODUTO
                    WHERE 
                       CDPRODUTO = NVL(:P_CDPRODUTO,CDPRODUTO)";

            $params = array(
                'P_CDPRODUTO' => $row
            ); 
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }  
    
    public function validaArvItem($row){
        try {
            $sql = "SELECT MAX( CDNVPRODUTO)
                    FROM
                       PRODUTO";
            ;
            
            $params = array(); 
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    public function getIdorigdestop($cdtipooper, $identrsaidop){
        try{
            $sql = "SELECT IDORIGDESTOP 
                    FROM TIPOOPERAC 
                    WHERE CDTIPOOPER = :P_CDTIPOOPER
                            AND IDENTRSAIDOP = :P_IDENTRSAIDOP";
            
            $params = array(
                'P_CDTIPOOPER'   => $cdtipooper, 
                'P_IDENTRSAIDOP' => $identrsaidop
            );
            //var_dump($params);die;
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    public function buscaPrecoPed($pedido, $item){
        try{
            $sql = 'SELECT BUSCA_PRECO_PED(:P_CDFILIAL, :P_CDARVPROD, :P_CDCLIENTE, :P_CDPRECOBASE, :P_DTENTPED, :P_CDMODPGTO) AS PRECOBASE FROM DUAL';
            
            $params = array(
                'P_CDFILIAL'   => $pedido[0]['CDFILIAL'], 
                'P_CDARVPROD'  => $item['CDARVPROD'],
                'P_CDCLIENTE'  => $pedido[0]['CDCLIENTE'], 
                'P_CDPRECOBASE'=> $pedido[0]['CDPRECOBASE'], 
                'P_DTENTPED'   => $pedido[0]['DTENTPED'], 
                'P_CDMODPGTO'  => $pedido[0]['CDMODPGTO']
            );
            //var_dump($params);die;
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    const INSERT_TABELA_PEDIDO= "INSERT INTO PEDIDO
                  (CDFILIAL, NRPEDIDO, IDSITPEDIDO, CDCLIENTE, IDPRIOPED, DTPEDIDO, DTENTPED,
                   CDMODPGTO, DTHRLANPED, CDTIPOOPER, CDOPELANPED, IDPEDIDOIMPR, DSPEDCLI,
                   DSPEDVEN, DTVALIPED, IDTIPOPEDIDO, CDVENDEDOR, NREXPEDIDO, CDPRECOBASE,
                   IDORIGPED, IDTPFORPGTPED, NRDIAPGFIXPED, NRPARCPGTOPED, IDUTDIAPAGPED,
                   CDFORMPGTO, IDENTRSAIDOP, CDTIPTRANS, IDPGFRETTRAN, NRINSJURTRAN,
                   IDTPIJURTRAN, CDVIATURA, CDMOTORISTA, CDMOTORAJUDA)
                VALUES
                  (:P_CDFILIAL, :P_NRPEDIDO, :P_IDSITPEDIDO, :P_CDCLIENTE, :P_IDPRIOPED, :P_DTPEDIDO,
                   :P_DTENTPED, :P_CDMODPGTO, :P_DTHRLANPED, :P_CDTIPOOPER, :P_CDOPELANPED, :P_IDPEDIDOIMPR,
                   :P_DSPEDCLI, :P_DSPEDVEN, :P_DTVALIPED, :P_IDTIPOPEDIDO, :P_CDVENDEDOR, :P_NREXPEDIDO, :P_CDPRECOBASE,
                   :P_IDORIGPED, :P_IDTPFORPGTPED, :P_NRDIAPGFIXPED, :P_NRPARCPGTOPED, :P_IDUTDIAPAGPED,
                   :P_CDFORMPGTO, :P_IDENTRSAIDOP,  :P_CDTIPTRANS, :P_IDPGFRETTRAN, :P_NRINSJURTRAN,
                   :P_IDTPIJURTRAN, :P_CDVIATURA, :P_CDMOTORISTA, :P_CDMOTORAJUDA)";
    
    public function insereTabelaPedido($pedido){
        try{
            $params = array(
                'P_CDFILIAL'     =>$pedido[0]['CDFILIAL'],
                'P_NRPEDIDO'     =>$pedido[0]['NRPEDIDO'],
                'P_IDSITPEDIDO'  =>$pedido[0]['IDSITPEDIDO'],
                'P_CDCLIENTE'    =>$pedido[0]['CDCLIENTE'],
                'P_IDPRIOPED'    =>$pedido[0]['IDPRIOPED'],
                'P_DTPEDIDO'     =>$pedido[0]['DTPEDIDO'],
                'P_DTENTPED'     =>$pedido[0]['DTENTPED'],
                'P_CDMODPGTO'    =>$pedido[0]['CDMODPGTO'],
                'P_DTHRLANPED'   =>$pedido[0]['DTHRLANPED'],
                'P_CDTIPOOPER'   =>$pedido[0]['CDTIPOOPER'],
                'P_CDOPELANPED'  =>$pedido[0]['CDOPELANPED'],
                'P_IDPEDIDOIMPR' =>$pedido[0]['IDPEDIDOIMPR'],
                'P_DSPEDCLI'     =>$pedido[0]['DSPEDCLI'],
                'P_DSPEDVEN'     =>$pedido[0]['DSPEDVEN'],
                'P_DTVALIPED'    =>$pedido[0]['DTVALIPED'],
                'P_IDTIPOPEDIDO' =>$pedido[0]['IDTIPOPEDIDO'],
                'P_CDVENDEDOR'   =>$pedido[0]['CDVENDEDOR'],
                'P_NREXPEDIDO'   =>$pedido[0]['NREXPEDIDO'],
                'P_CDPRECOBASE'  =>$pedido[0]['CDPRECOBASE'],
                'P_IDORIGPED'    =>$pedido[0]['IDORIGPED'],
                'P_IDTPFORPGTPED'=>$pedido[0]['IDTPFORPGTPED'],
                'P_NRDIAPGFIXPED'=>$pedido[0]['NRDIAPGFIXPED'],
                'P_NRPARCPGTOPED'=>$pedido[0]['NRPARCPGTOPED'],
                'P_IDUTDIAPAGPED'=>$pedido[0]['IDUTDIAPAGPED'],
                'P_CDFORMPGTO'   =>$pedido[0]['CDFORMPGTO'],
                'P_IDENTRSAIDOP' =>$pedido[0]['IDENTRSAIDOP'],
                'P_CDTIPTRANS'   =>$pedido[0]['CDTIPTRANS'],
                'P_IDPGFRETTRAN' =>$pedido[0]['IDPGFRETTRAN'],
                'P_NRINSJURTRAN' =>$pedido[0]['NRINSJURTRAN'],
                'P_IDTPIJURTRAN' =>$pedido[0]['IDTPIJURTRAN'],
                'P_CDVIATURA'    =>null,
                'P_CDMOTORISTA'  =>null,
                'P_CDMOTORAJUDA' =>null
            );

            return $this->entityManager->getConnection()->executeQuery(self::INSERT_TABELA_PEDIDO, $params);

        }catch(\Exception $e){
            throw $e;
            $this->entityManager->rollback();
        }
    }
    
   
    
    public function getDadosEntrega($cliente, $tipoEnd){
        try {
            $sql = "SELECT EN.CDCLIENTE, EN.NRSEQEND,  EN.IDTPENDECLIE, EN.CDPAIS, EN.SGESTADO,
                       EN.CDMUNICIPIO,EN.DSENDECLIE, EN.NMBAIRCLIE, EN.NRCEPCLIE,
                       MU.NMMUNICIPIO, PA.NMPAIS, ES.NMESTADO, C.NMRAZSOCCLIE AS NMRAZSOCPED,
                       EN.NRTELECLIE, C.NRINSJURCLIE, C.NRINSESTCLIE
                    FROM ENDECLIE EN,
                         MUNICIPIO MU, 
                         PAIS PA, 
                         ESTADO ES, 
                         CLIENTE C
                    WHERE (EN.CDCLIENTE    = :P_CDCLIENTE)
                           AND (EN.CDPAIS       = ES.CDPAIS)
                           AND (EN.SGESTADO     = ES.SGESTADO)
                           AND (EN.IDTPENDECLIE = NVL(:P_IDTPENDECLIE,'P'))
                           AND (EN.CDPAIS       = MU.CDPAIS)
                           AND (EN.SGESTADO     = MU.SGESTADO)
                           AND (EN.CDMUNICIPIO  = MU.CDMUNICIPIO)
                           AND (EN.CDPAIS       = PA.CDPAIS)
                           AND (EN.CDCLIENTE    = C.CDCLIENTE)";
            ;
            
            $params = array(
                'P_CDCLIENTE' => $cliente,
                'P_IDTPENDECLIE' => $tipoEnd
            ); 
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
     const INSERT_DADOSENDPED = "INSERT INTO DADOSENDPED (CDFILIAL, NRPEDIDO, NMRAZSOCPED, DSENDEPED, NRCEPPED,
                                             NRTELEPED, NRINSJURPED, NRINSESTPED, NMPAISPED,
                                             SGESTADOPED, NMMUNIPED, NMBAIRROPED, CDPAISPED, NMESTADOPED, CDMUNICIPIOPED,
                                             CDCLIENTE, IDTPENDECLIE, NRSEQEND)
                                VALUES (:CDFILIAL, :NRPEDIDO, :NMRAZSOCPED, :DSENDEPED, :NRCEPPED, :NRTELEPED,
                                        :NRINSJURPED, :NRINSESTPED, :NMPAISPED,
                                        :SGESTADOPED, :NMMUNIPED, :NMBAIRROPED, :CDPAISPED, :NMESTADOPED, :CDMUNICIPIOPED,
                                        :CDCLIENTE, :IDTPENDECLIE, :NRSEQEND)";
            
    public function insertDadosEntr($endEntrega, $pedido){
        try {

            $params = array(
                'CDFILIAL'       =>$pedido[0]['CDFILIAL'],
                'NRPEDIDO'       =>$pedido[0]['NRPEDIDO'],
                'NMRAZSOCPED'    =>$endEntrega['NMRAZSOCPED'],
                'DSENDEPED'      =>$endEntrega['DSENDECLIE'],
                'NRCEPPED'       =>$endEntrega['NRCEPCLIE'], 
                'NRTELEPED'      =>$endEntrega['NRTELECLIE'],
                'NRINSJURPED'    =>$endEntrega['NRINSJURCLIE'],
                'NRINSESTPED'    =>$endEntrega['NRINSESTCLIE'],
                'NMPAISPED'      =>$endEntrega['NMPAIS'],
                'SGESTADOPED'    =>$endEntrega['SGESTADO'],
                'NMMUNIPED'      =>$endEntrega['NMMUNICIPIO'],
                'NMBAIRROPED'    =>$endEntrega['NMBAIRCLIE'],
                'CDPAISPED'      =>$endEntrega['CDPAIS'],
                'NMESTADOPED'    =>$endEntrega['NMESTADO'],
                'CDMUNICIPIOPED' =>$endEntrega['CDMUNICIPIO'],
                'CDCLIENTE'      =>$endEntrega['CDCLIENTE'],
                'IDTPENDECLIE'   =>$endEntrega['IDTPENDECLIE'], 
                'NRSEQEND'       =>$endEntrega['NRSEQEND']
            );            
            
            return $this->entityManager->getConnection()->executeQuery(self::INSERT_DADOSENDPED, $params);
            
        } catch (\Exception $e) {
            throw $e;
            $this->entityManager->rollback();
        }
    }
    
    const INSERT_ITENSPEDIDO= "INSERT INTO ITEMPEDIDO
                          (NRPEDIDO, NRITEMPED, CDPRODUTO, QTITEMPED, VRPREUNITPED, VRDESITPED,
                           CDFILIAL, QTEXPITEMPED, IDSITITEMPED, VRPEDEMAXIPE, CDOPERADOR, DTENTITPED, IDITPEDDIFER)
                        VALUES
                          (:NRPEDIDO, :NRITEMPED, :CDPRODUTO, :QTITEMPED, :VRPREUNITPED, :VRDESITPED,
                           :CDFILIAL, :QTEXPITEMPED, :IDSITITEMPED, :VRPEDEMAXIPE, :CDOPERADOR,
                           :DTENTITPED, :IDITPEDDIFER)";
                            
    
    public function insereItenspedido($item, $pedido){
        try{
           
            $params = array(
                'NRPEDIDO'     =>$pedido[0]['NRPEDIDO'],
                'NRITEMPED'    =>$item['NRITEMPED'],
                'CDPRODUTO'    =>$item['CDPRODUTO'],
                'QTITEMPED'    =>$item['QTITEMPED'],
                'VRPREUNITPED' =>$item['VRPREUNITPED'],
                'VRDESITPED'   =>$item['VRDESITPED'],
                'CDFILIAL'     =>$pedido[0]['CDFILIAL'],
                'QTEXPITEMPED' =>$item['QTEXPITEMPED'],
                'IDSITITEMPED' =>$item['IDSITITEMPED'],
                'VRPEDEMAXIPE' =>null,
                'CDOPERADOR'   =>$pedido[0]['CDOPERADOR'],
                'DTENTITPED'   =>$pedido[0]['DTENTPED'],
                'IDITPEDDIFER' => 'N'
                
            );

            return $this->entityManager->getConnection()->executeQuery(self::INSERT_ITENSPEDIDO, $params);
            
        }catch(\Exception $e){
            throw $e;
            $this->entityManager->rollback();
        }
    }
    public function getPrazo($row){
        try{
            $sql = "SELECT
                   NRDIASMODPGTO,
                   CDMODPGTO,
                   VRPERCMODPGTO
                FROM
                   PRAZOMODPGTO
                WHERE
                   CDMODPGTO = CDMODPGTO AND
                   CDMODPGTO = :P_CDMODPGTO
                ORDER BY
                   CDMODPGTO, NRDIASMODPGTO";

            $params = array(
                'P_CDMODPGTO' =>$row
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    const INSERT_PRAZOS= "INSERT INTO PRAZOPGTOPED
                        (NRPEDIDO, NRDIASPGTOPED, VRPERCPGTOPED, CDFILIAL, DTINCLUSAO, DTULTATU, CDOPERINCLUSAO, CDOPERULTATU)
                    VALUES
                       (:P_NRPEDIDO, :P_NRDIASPGTOPED, :P_VRPERCPGTOPED, :P_CDFILIAL, :P_DTINCLUSAO, :P_DTULTATU, :P_CDOPERINCLUSAO, :P_CDOPERULTATU)";
    
    public function inserePrazo($pedido, $prazo){
        try{
            $params = array(
                'P_NRPEDIDO'        =>$pedido[0]['NRPEDIDO'],
                'P_NRDIASPGTOPED'   =>$prazo['NRDIASMODPGTO'],
                'P_VRPERCPGTOPED'   =>$prazo['VRPERCMODPGTO'],
                'P_CDFILIAL'        =>$pedido[0]['CDFILIAL'],
                'P_DTINCLUSAO'      =>$pedido[0]['DTPEDIDO'], 
                'P_DTULTATU'        =>$pedido[0]['DTPEDIDO'], 
                'P_CDOPERINCLUSAO'  =>$pedido[0]['CDOPERADOR'], 
                'P_CDOPERULTATU'    =>$pedido[0]['CDOPERADOR']
                
            );
            
             return $this->entityManager->getConnection()->executeQuery(self::INSERT_PRAZOS, $params);
             
        }catch(\Exception $e){
            throw $e;
            $this->entityManager->rollback();
        }
    }
    
      public function geraContaPrevista($cdfilial, $nrpedido, $idoperacao) {
        try {

            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'GERA_CONTARECEBER_PREVISTA');
            $procedure->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT,  $cdfilial  , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRPEDIDO',    Param::PARAM_INPUT,  $nrpedido  , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDOPERACAO',  Param::PARAM_INPUT,  $idoperacao, Param::PARAM_TYPE_STR));
            return $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    /*public function gravaItemPedNovo($pedido, $prxPedido, $item, $vrpreunitpred, $cdfatpreco){
        try {

            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'GRAVA_ITEMPEDIDO');
            $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT,  $pedido['CDFILIAL']       , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRPEDIDO',        Param::PARAM_INPUT,  $prxPedido['NOVOCODIGO']  , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODUTO' ,      Param::PARAM_INPUT,  $item['CDPRODUTO']        , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_QTITEMPED',       Param::PARAM_INPUT,  $item['QTITEMPED']        , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRPREUNITPED',    Param::PARAM_INPUT,  $vrpreunitpred            , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFATPRECO' ,     Param::PARAM_INPUT,  $cdfatpreco               , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRDESITPED',      Param::PARAM_INPUT,  ''                        , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRPESOITPED',     Param::PARAM_INPUT,  ''                        , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRPRTABITPED',    Param::PARAM_INPUT,  ''                        , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRPEDEMAXIPE',    Param::PARAM_INPUT,  ''                        , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTENTITPED',      Param::PARAM_INPUT,  $pedido['DTENTPED']       , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTENTPED',        Param::PARAM_INPUT,  $pedido['DTENTPED']       , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRITEMORCAMENTO', Param::PARAM_INPUT,  ''                        , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMENSERRO',      Param::PARAM_OUTPUT, ''                        , Param::PARAM_TYPE_STR,100));
            $procedure->addParam(new Param('P_NRITEMPED',       Param::PARAM_OUTPUT, ''                        , Param::PARAM_TYPE_STR,100));
            return $procedure->execute(); 
            
        } catch (\Exception $e) {
            throw $e;
        }
    }*/
    
    const DELETE_RESERPRODPED = "DELETE FROM RESERPRODPED
                    WHERE CDPRODUTO = :P_CDPRODUTO
                    AND CDFILIAL = :P_CDFILIAL
                    AND NRPEDIDO = :P_NRPEDIDO
                    AND NRITEMPED = :P_NRITEMPED";
    
    public function deleteReserprodped($cdfilial, $nrpedido, $nritemped, $cdproduto) {
        try {
            $params = array(
                'P_CDFILIAL' => $cdfilial,
                'P_NRPEDIDO' => $nrpedido,
                'P_NRITEMPED' => $nritemped,
                'P_CDPRODUTO' => $cdproduto
            );
            
            $this->entityManager->getConnection()->executeQuery(self::DELETE_RESERPRODPED, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
}